==> snaplib-2.0.1-py2-none-any.whl: snaplib/DataModeCodec.py <==
# (c) Copyright 2008, Synapse
"""
Functions to encode and decode SNAP data mode packets
"""
__docformat__ = "plaintext en"


import binascii

import SnapConstantsHelper
import SnapEncryptions
import sequence


class DataModeEncodeError(Exception):
    def __init__(self, descr):
        self.descr = descr


class DataModeDescriptor(object):
    def __init__(self, crypto=None):
        self.srcAddr = None    # 3-byte string
        self.dstAddr = None    # 3-byte string or 2-byte multicast group
        self.data = ""
        self.isMulticast = False
        self.ttl = 0
        self.final_dst_addr = None
        self.seq = None
        self.last_hop_addr = None
        self.crypto = crypto

    def __str__(self):
        return "%s: %s" % (binascii.hexlify(self.dstAddr), self.data)

    def get_encoded_data(self):
        return encode(self, crypto=self.crypto)

    encoded_data = property(get_encoded_data)


def decode(pkt, crypto=None):
    """
    Decodes a PTP Data Mode packet from a SNAP device

    pkt -- raw data packet from packet serial
    """
    desc = DataModeDescriptor(crypto=crypto)
    desc.seq = pkt[SnapConstantsHelper.RPC_SEQ_OFS]

    pkt = SnapEncryptions.decode_pkt(pkt, crypto)
    pid = ord(pkt[SnapConstantsHelper.RPC_PID_OFS])

    if pid == SnapConstantsHelper.PID_DM_BCAST or pid == (SnapConstantsHelper.PID_DM_BCAST | SnapConstantsHelper.PID_ENCRYPTION_MASK):
        desc.isMulticast = True
        desc.srcAddr = pkt[SnapConstantsHelper.DMBCAST_ORIG_SRC_OFS : SnapConstantsHelper.DMBCAST_ORIG_SRC_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN]
        desc.dstAddr = pkt[SnapConstantsHelper.DMPTP_DST_OFS : SnapConstantsHelper.DMPTP_DST_OFS+2]
        desc.data = pkt[SnapConstantsHelper.DMPTP_ORIG_SRC_OFS:]
        desc.ttl = ord(pkt[SnapConstantsHelper.DMBAST_TTL_OFS])
    elif pid == SnapConstantsHelper.PID_DM_PTP or pid == (SnapConstantsHelper.PID_DM_PTP | SnapConstantsHelper.PID_ENCRYPTION_MASK):
        desc.dstAddr = pkt[SnapConstantsHelper.DMPTP_DST_OFS : SnapConstantsHelper.DMPTP_DST_OFS+SnapConstantsHelper.SNAP2_ADDR_LEN]
        desc.srcAddr = pkt[SnapConstantsHelper.DMPTP_ORIG_SRC_OFS : SnapConstantsHelper.DMPTP_ORIG_SRC_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN]
        desc.last_hop_addr = pkt[SnapConstantsHelper.DMPTP_FINALDST_OFS : SnapConstantsHelper.DMPTP_FINALDST_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN]
        desc.data = pkt[SnapConstantsHelper.DMPTP_ORIG_SRC_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN:]
        desc.final_dst_addr = pkt[SnapConstantsHelper.DMPTP_FINALDST_OFS : SnapConstantsHelper.DMPTP_FINALDST_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN]
    else:
        return None

    return desc

def encode(descriptor, crypto=None):
    """
    Encode a PTP Data Mode packet for output to a SNAP device

    descriptor -- A DataModeDescriptor object
    """
    assert isinstance(descriptor, DataModeDescriptor)
    last_hop_addr = descriptor.srcAddr

    if descriptor.ttl != 0:
        pid = SnapConstantsHelper.PID_DM_BCAST
        ttl = "%c" % descriptor.ttl
        finalDstAddr = ''
    else:
        pid = SnapConstantsHelper.PID_DM_PTP
        ttl = ''
        finalDstAddr = descriptor.final_dst_addr
        if finalDstAddr is None:
            finalDstAddr = descriptor.dstAddr

    if descriptor.seq is not None:
        txSequence = descriptor.seq
    else:
        txSequence = sequence.get_number(descriptor.srcAddr)

    if descriptor.last_hop_addr is not None:
        last_hop_addr = descriptor.last_hop_addr

    # Len(reserved) | PID | DstAddr (or mcast: Group and TTL) | SrcAddr | Sequence | Final DstAddr(not in mcast) | Orig SrcAddr | data
    pkt = "%c%s%s%s%c%s%s%s" % (pid, descriptor.dstAddr, ttl, last_hop_addr, txSequence, finalDstAddr, descriptor.srcAddr, descriptor.data)

    try:
        pkt = "%c%s" % (len(pkt), pkt) # Prepend length
    except OverflowError:
        raise DataModeEncodeError("Packet is greater than 255 bytes")

    if len(pkt) > SnapConstantsHelper.getMaxPacketSize():
        raise DataModeEncodeError('The packet is too large to encode (%d bytes, max is %d)' % (len(pkt), SnapConstantsHelper.getMaxPacketSize()))

    pkt = SnapEncryptions.encode_pkt(pkt, crypto)

    return pkt

==> snaplib-2.3.1-py2-none-any.whl: snaplib/DataModeCodec.py <==
# (c) Copyright 2008-2015 Synapse Wireless, Inc.
"""
Functions to encode and decode SNAP data mode packets
"""
__docformat__ = "plaintext en"


import binascii

from snappy import SnapConstantsHelper
import SnapEncryptions
import sequence


class DataModeEncodeError(Exception):
    def __init__(self, descr):
        self.descr = descr


class DataModeDescriptor(object):
    def __init__(self, crypto=None):
        self.srcAddr = None    # 3-byte string
        self.dstAddr = None    # 3-byte string or 2-byte multicast group
        self.data = ""
        self.isMulticast = False
        self.ttl = 0
        self.final_dst_addr = None
        self.seq = None
        self.last_hop_addr = None
        self.crypto = crypto

    def __str__(self):
        return "%s: %s" % (binascii.hexlify(self.dstAddr), self.data)

    def get_encoded_data(self):
        return encode(self, crypto=self.crypto)

    encoded_data = property(get_encoded_data)


def decode(pkt, crypto=None):
    """
    Decodes a PTP Data Mode packet from a SNAP device

    pkt -- raw data packet from packet serial
    """
    desc = DataModeDescriptor(crypto=crypto)
    desc.seq = pkt[SnapConstantsHelper.RPC_SEQ_OFS]

    pkt = SnapEncryptions.decode_pkt(pkt, crypto)
    pid = ord(pkt[SnapConstantsHelper.RPC_PID_OFS])

    if pid == SnapConstantsHelper.PID_DM_BCAST or pid == (SnapConstantsHelper.PID_DM_BCAST | SnapConstantsHelper.PID_ENCRYPTION_MASK):
        desc.isMulticast = True
        desc.srcAddr = pkt[SnapConstantsHelper.DMBCAST_ORIG_SRC_OFS : SnapConstantsHelper.DMBCAST_ORIG_SRC_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN]
        desc.dstAddr = pkt[SnapConstantsHelper.DMPTP_DST_OFS : SnapConstantsHelper.DMPTP_DST_OFS+2]
        desc.data = pkt[SnapConstantsHelper.DMPTP_ORIG_SRC_OFS:]
        desc.ttl = ord(pkt[SnapConstantsHelper.DMBAST_TTL_OFS])
    elif pid == SnapConstantsHelper.PID_DM_PTP or pid == (SnapConstantsHelper.PID_DM_PTP | SnapConstantsHelper.PID_ENCRYPTION_MASK):
        desc.dstAddr = pkt[SnapConstantsHelper.DMPTP_DST_OFS : SnapConstantsHelper.DMPTP_DST_OFS+SnapConstantsHelper.SNAP2_ADDR_LEN]
        desc.srcAddr = pkt[SnapConstantsHelper.DMPTP_ORIG_SRC_OFS : SnapConstantsHelper.DMPTP_ORIG_SRC_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN]
        desc.last_hop_addr = pkt[SnapConstantsHelper.DMPTP_FINALDST_OFS : SnapConstantsHelper.DMPTP_FINALDST_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN]
        desc.data = pkt[SnapConstantsHelper.DMPTP_ORIG_SRC_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN:]
        desc.final_dst_addr = pkt[SnapConstantsHelper.DMPTP_FINALDST_OFS : SnapConstantsHelper.DMPTP_FINALDST_OFS + SnapConstantsHelper.SNAP2_ADDR_LEN]
    else:
        return None

    return desc

def encode(descriptor, crypto=None):
    """
    Encode a PTP Data Mode packet for output to a SNAP device

    descriptor -- A DataModeDescriptor object
    """
    assert isinstance(descriptor, DataModeDescriptor)
    last_hop_addr = descriptor.srcAddr

    if descriptor.ttl != 0:
        pid = SnapConstantsHelper.PID_DM_BCAST
        ttl = "%c" % descriptor.ttl
        finalDstAddr = ''
    else:
        pid = SnapConstantsHelper.PID_DM_PTP
        ttl = ''
        finalDstAddr = descriptor.final_dst_addr
        if finalDstAddr is None:
            finalDstAddr = descriptor.dstAddr

    if descriptor.seq is not None:
        txSequence = descriptor.seq
    else:
        txSequence = sequence.get_number(descriptor.srcAddr)

    if descriptor.last_hop_addr is not None:
        last_hop_addr = descriptor.last_hop_addr

    # Len(reserved) | PID | DstAddr (or mcast: Group and TTL) | SrcAddr | Sequence | Final DstAddr(not in mcast) | Orig SrcAddr | data
    pkt = "%c%s%s%s%c%s%s%s" % (pid, descriptor.dstAddr, ttl, last_hop_addr, txSequence, finalDstAddr, descriptor.srcAddr, descriptor.data)

    try:
        pkt = "%c%s" % (len(pkt), pkt) # Prepend length
    except OverflowError:
        raise DataModeEncodeError("Packet is greater than 255 bytes")

    if len(pkt) > SnapConstantsHelper.getMaxPacketSize():
        raise DataModeEncodeError('The packet is too large to encode (%d bytes, max is %d)' % (len(pkt), SnapConstantsHelper.getMaxPacketSize()))

    pkt = SnapEncryptions.encode_pkt(pkt, crypto)

    return pkt
